<?php

namespace App\Http\Controllers;

use App\domain\amadeus\AddPNRMultiElementAction;
use App\domain\amadeus\createFOPAction;
use App\domain\amadeus\CreateTSTFromPricing;
use App\domain\amadeus\DocIssuanceFlowAction;
use App\domain\amadeus\FareInformativePricingAction;
use App\domain\amadeus\FlightSearchAction;
use App\domain\amadeus\PaymentFlowAction;
use App\domain\amadeus\PNRAddMultiElementsSecondAction;
use App\domain\amadeus\QueryCombinedOffersAction;
use App\domain\amadeus\QueryCombinedOffersPageAction;
use App\domain\amadeus\QueryDocIssuanceIssueTicketAction;
use App\domain\amadeus\QueryFareCheckRulesAction;
use App\domain\amadeus\QueryFareInformativePriceAction;
use App\domain\amadeus\QueryFareMasterPricerTravelBoardSearchAction;
use App\domain\amadeus\queryFarePricePNRWithBookingClassAction;
use App\domain\amadeus\QueryPNRRetrieveAction;
use App\domain\amadeus\QuerySecuritySignOut;
use App\domain\amadeus\QueryTravelersAction;
use App\domain\amadeus\SelFromRecommendationAction;
use App\domain\travelport\QueryOffersPageAction;
use App\domain\worldpay\QueryWorldpayPaymentStatusAction;
use App\Traits\FileStorageTrait;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Log;
use Throwable;

class AmaFlightOfferSearchController extends Controller
{
    public function index(Request $request, FlightSearchAction $action): JsonResponse
    {
        try {
            $response = $action->execute(
                $request->input('passengerCriteria'),
                $request->input('searchCriteriaFlight'),
                null
            );

            return response()->json([
                'productOffers' => $response,
                'amadeusSessionId' => session("amadeus-session-id"),
                'amadeusSecurityToken' => session("amadeus-security-token"),
            ]);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->index : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function queryFareMasterPricer(Request $request, QueryFareMasterPricerTravelBoardSearchAction $action): JsonResponse
    {
        try {
            $response = $action->execute(
                $request->input('passengerCriteria'),
                $request->input('searchCriteriaFlight'),
                null
            );

            return response()->json([
                'productOffers' => $response,
                'amadeusSessionId' => session("amadeus-session-id"),
                'amadeusSecurityToken' => session("amadeus-security-token"),
            ]);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->index : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function queryCombinedOffers(Request $request, QueryCombinedOffersAction $action): JsonResponse
    {
        try {
            $searchModifiers = [
                "CabinPreference" => $request->input('CabinPreference'),
                "Source" => $request->input('source'),
                "Carriers" => $request->input('carrier')
            ];

            $response = $action->execute(
                $request->input('passengerCriteria'),
                $request->input('searchCriteriaFlight'),
                $searchModifiers
            );

            $combinedOffers = [
                'productOffers' => $response['productOffers'],
                'total_combined_offers' => $response['amaTotalCombinedOffers'],
                'offers_request_id' => $response['amaOffersRequestId'],
                'offersGroup' => $response['offersGroup'],
                'errorMessage' => $response['errorMessage'],
                'amadeusSessionId' => session("amadeus-session-id"),
                'amadeusSecurityToken' => session("amadeus-security-token"),
            ];

            FileStorageTrait::storeInFile(
                "json",
                "0-ama_combined_offers_final_response" . Carbon::now()->timestamp,
                json_encode($combinedOffers)
            );

            return response()->json($combinedOffers);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->queryCombinedOffers : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function getOffersPage(Request $request, QueryCombinedOffersPageAction $action): JsonResponse
    {
        try {
            $offers = $action->execute(
                $request->input('requestId'),
                $request->input('page'),
            );

            return response()->json([
                'productOffers' => $offers,
                //                'original_response' => $searchResults
            ]);
        } catch (\Exception $exception) {
            Log::error("FlightOfferSearchController->getOffersPage : " . $exception->getMessage());
            return response()->json(['error_message' => $exception->getMessage()], Response::HTTP_BAD_REQUEST);
        }
    }

    public function fareInformativePricingWithoutPNR(Request $request, QueryFareInformativePriceAction $action): JsonResponse
    {
        FileStorageTrait::storeInFile("json", "Fare Informative Front End Request" . Carbon::now()->timestamp, json_encode($request->all()));

        try {
            $response = $action->execute(
                $request->input('passengerCriteria'),
                $request->input('searchCriteriaFlight'),
                $request->input('selectedSegments'),
                $request->input('recommendation'),
            );

            return response()->json([
                "amadeusReply" => $response,
                'amadeusSessionId' => session("amadeus-session-id"),
                'amadeusSecurityToken' => session("amadeus-security-token"),
                'fareInformativeCombinedSegments' => session('fare-informative-combined-segments'),
                'otherCompany' => session('other-company')
            ]);
        } catch (Exception $e) {
            Log::error("AmaFlightOfferSearchController->fareInformativePricingWithoutPNR : " . $e->getMessage());
            return response()->json(
                [
                    'error_message' => $e->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        } catch (Throwable $e) {
            Log::error("Fare Informative Throwable  : " . $e->getMessage());
            return response()->json(
                [
                    'error_message' => $e->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function fareCheckRules(Request $request, QueryFareCheckRulesAction $action): JsonResponse
    {
        try {
            $response = $action->execute(
                $request->input('amadeusSessionId'),
                $request->input('amadeusSecurityToken'),
            );

            return response()->json([
                "amadeusReply" => $response
            ]);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->fareCheckRules : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function securitySignOut(Request $request, QuerySecuritySignOut $action): JsonResponse
    {
        try {
            $response = $action->execute(
                $request->input('amadeusSessionId'),
                $request->input('amadeusSecurityToken'),
            );

            return response()->json([
                "amadeusReply" => $response
            ]);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->securitySignOut : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function selRecommendation(Request $request, SelFromRecommendationAction $action): JsonResponse
    {
        FileStorageTrait::storeInFile("json", "Sel From Recommendation Front End Request", json_encode($request->all()));

        try {
            $response = $action->execute(
                $request->input('passengerCriteria'),
                $request->input('searchCriteriaFlight'),
                $request->input('selectedSegments'),
                $request->input('recommendation'),
            );

            return response()->json([
                "amadeusReply" => $response,
                'amadeusSessionId' => session("amadeus-session-id"),
                'amadeusSecurityToken' => session("amadeus-security-token"),
            ]);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->selRecommendation : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function addTravelerInPNRAddMultiElements(Request $request, AddPNRMultiElementAction $action): JsonResponse
    {
        FileStorageTrait::storeInFile("json", "Add-travelers-paylod" . Carbon::now()->timestamp, json_encode($request->all()));

        try {
            $response = $action->execute(
                $request->input('traveller'),
                $request->input('amadeusSessionId'),
                $request->input('amadeusSecurityToken')
            );

            return response()->json($response);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->addPNRMultiElements : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function createFOP(Request $request, createFOPAction $action): JsonResponse
    {
        try {
            $response = $action->execute(
                $request->input('payType'),
                $request->input('amadeusSessionId'),
                $request->input('amadeusSecurityToken')
            );

            return response()->json($response);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->createFOP : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function queryFarePricePNRWithBookingClass(Request $request, queryFarePricePNRWithBookingClassAction $action): JsonResponse
    {
        try {
            $response = $action->execute(
                $request->input('otherCompany'),
                $request->input('amadeusSessionId'),
                $request->input('amadeusSecurityToken')
            );

            return response()->json($response);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->createFOP : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function createTSTFromPricing(Request $request, CreateTSTFromPricing $action): JsonResponse
    {
        try {
            $response = $action->execute(
                $request->input('passengerCriteria'),
                $request->input('amadeusSessionId'),
                $request->input('amadeusSecurityToken')
            );

            return response()->json($response);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->createTSTFromPricing : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function createPNRAddMultiElements(Request $request, PNRAddMultiElementsSecondAction $action): JsonResponse
    {
        try {
            $response = $action->execute(
                $request->input('amadeusSessionId'),
                $request->input('amadeusSecurityToken'),
                $request->input('reservationId')
            );

            return response()->json(['amadeusReply' => $response]);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->createPNRAddMultiElements : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function queryPNRRetrieve(Request $request, QueryPNRRetrieveAction $action): JsonResponse
    {
        try {
            $response = $action->execute($request->input('controlNumber'));

            return response()->json([
                "amadeusReply" => $response,
                'amadeusSessionId' => session("amadeus-session-id"),
                'amadeusSecurityToken' => session("amadeus-security-token"),
            ]);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->createPNRAddMultiElements : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function issueTicket(Request $request, QueryDocIssuanceIssueTicketAction $action): JsonResponse
    {
        try {
            $response = $action->execute(
                $request->input('amadeusSessionId'),
                $request->input('amadeusSecurityToken')
            );

            return response()->json([
                "amadeusReply" => $response,
            ]);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->createPNRAddMultiElements : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function paymentFlow(Request $request, PaymentFlowAction $action): JsonResponse
    {
        try {
            $response = $action->execute(
                $request->input('amadeusSessionId'),
                $request->input('amadeusSecurityToken'),
                $request->input('payType'),
                $request->input('otherCompany'),
                $request->input('passengerCriteria'),
                $request->input('reservationId'),
            );

            return response()->json([
                "amadeusReply" => $response,
            ]);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->paymentFlow : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function docIssuanceFlow(
        Request $request,
        DocIssuanceFlowAction $action,
        QueryWorldpayPaymentStatusAction $worldpayPaymentStatusAction

    ): JsonResponse {
        try {
            $orderId = $request->input('controlNumber');

            $isPaymentSuccess = $worldpayPaymentStatusAction->isPaymentSuccess($orderId);

            Log::warning("isPaymentSuccess or not: " . $isPaymentSuccess);

            if (!$isPaymentSuccess) {
                return response()->json([
                    'error_message' => "Payments not success.",
                ], Response::HTTP_FORBIDDEN);
            }

            $response = $action->execute($orderId);

            return response()->json([
                "amadeusReply" => $response,
            ]);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->docIssuanceFlow : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }

    public function queryTravelers(QueryTravelersAction $action): JsonResponse
    {
        try {
            $response = $action->execute();

            return response()->json($response);
        } catch (Exception $exception) {
            Log::error("AmaFlightOfferSearchController->queryTravelers : " . $exception->getMessage());
            return response()->json(
                [
                    'error_message' => $exception->getMessage(),
                ],
                Response::HTTP_BAD_REQUEST
            );
        }
    }
}
